/* This handles the transforming of links to
  popover and ajax loading of the bibliography
    via the popvers
*/
/* initialize all popover which have an element data-toggle="popover" */

$(function () {

  $(document).on('click', function (e) {
    var
      $popover,
      $target = $(e.target);

    //do nothing if there was a click on popover content
    if ($target.hasClass('popover') || $target.closest('.popover').length) {
      return;
    }

    $('[data-toggle="popover"]').each(function () {
      $popover = $(this);

      if (!$popover.is(e.target) &&
        $popover.has(e.target).length === 0 &&
        $('.popover').has(e.target).length === 0) {
        $popover.popover('hide');
      } else {
        //fixes issue described above
        $popover.popover('toggle');
      }
    });

    $('[data-toggle="popover"]').popover();
  })

});


/* handle the zotero bib popover click action */
$('.kmg-bib').click(function () {
  var e = $(this);
  if (e.data('loaded') !== true) {
    var multiple = false;
    var id = e.data('id');
    var ida = id.split(' ');
    var apiurl = null;
    if (ida.length == 1) {
      apiurl = "/exist/restxq/zotero/item/" + id;
    } else {
      apiurl = "/exist/restxq/zotero/items/" + ida.join(',');
      multiple = true;
    }
    $.ajax({
      url: apiurl,
      dataType: 'json',
      success: function (data) {
        e.data('loaded', true);
        var content = "";
        if (!multiple) {
          content = data.bib;
        } else {
          for (var key of Object.keys(data)) {
            content += data[key].bib;
          }
        }
        /* check if there is an additional Mollenhauer source entry and display it first
        then additional data from zotero */
        if (e.context.children[1].innerHTML != null && e.context.children[1].innerHTML != "") {
          e.attr('data-content', "<div class='zotero-content'>" + content + "</div><div class='zotero-suffix'>" + e.context.children[1].innerHTML + '</div>');
        } else {
          /* just add the zotero data */
          e.attr('data-content', "<div class='zotero-content'>" + content + '</div>');
        }
        var popover = e.data('bs.popover');
        popover.setContent();
        popover.$tip.addClass(popover.options.placement);
        var calculated_offset = popover.getCalculatedOffset(popover.options.placement, popover.getPosition(), popover.$tip[0].offsetWidth, popover.$tip[0].offsetHeight);
        popover.applyPlacement(calculated_offset, popover.options.placement);
      },
      error: function (jqXHR, textStatus, errorThrown) {
        e.attr('data-content', 'Daten konnten nicht geladen werden');
        var popover = e.data('bs.popover');
        popover.setContent();
      }
    });
  }
});

$('.kmg-pnd').click(function () {
  var e = $(this);
  if (e.data('loaded') !== true) {
    var id = e.data('id') + '';
    var ids = id.split(' ');
    var content = "";
    var deferreds = [];
    $.each(ids, function (index, pndid) {
      deferreds.push(
        $.get('https://ref.de.dariah.eu/pndsearch/json?id=' + pndid, function (res) {
          content += renderGNDData(res, pndid);
        })
      );
    });
    $.when.apply($, deferreds).then(function () {
      e.data('loaded', true);
      e.attr('data-content', content);

      var popover = e.data('bs.popover');
      popover.setContent();
      popover.$tip.addClass(popover.options.placement);
      var calculated_offset = popover.getCalculatedOffset(popover.options.placement, popover.getPosition(), popover.$tip[0].offsetWidth, popover.$tip[0].offsetHeight);
      popover.applyPlacement(calculated_offset, popover.options.placement);
    }).fail(function () {
      e.attr('data-content', 'Daten konnten nicht geladen werden');
      var popover = e.data('bs.popover');
      popover.setContent();
    });

  }

});

function renderGNDData(data, id) {
  var html = '<div class="gnd-table">'

  html += renderGNDDataRow('Name:', data.entry.preferredNameForThePerson);
  if (data.entry.si != null && Array.isArray(data.entry.si)) {
    var alternateNames = "";
    for (i = 1; i < data.entry.si.length; i++) {
      if (i == 4) {
        alternateNames += "[...]"
        break;
      }
      alternateNames += data.entry.si[i] + "<br/>";
    }
    html += renderGNDDataRow('Andere Namen:', alternateNames);
  }
  if (data.entry.variantNameForThePerson != null && Array.isArray(data.entry.variantNameForThePerson)) {
    var variants = "";
    for (i = 1; i < data.entry.variantNameForThePerson.length; i++) {
      if (i == 4) {
        variants += "[...]"
        break;
      }
      variants += data.entry.variantNameForThePerson[i] + "<br/>";
    }
    html += renderGNDDataRow('Varianten:', variants);
  }

  html += renderGNDDataRow('Weitere Angaben:', data.entry.biographicalInformation);
  html += renderGNDDataRow('Räumlicher Bezug:', data.entry.placeOfResidence);
  html += renderGNDDataRow('Geburtsdatum:', formatDate(data.entry.dateOfBirth));
  html += renderGNDDataRow('Geburtsort:', data.entry.placeOfBirth);
  html += renderGNDDataRow('Sterbedatum:', formatDate(data.entry.dateOfDeath));
  html += renderGNDDataRow('Sterbeort:', data.entry.placeOfDeath);
  html += renderGNDDataRow('GND Eintrag:', '<a href="http://d-nb.info/gnd/' + id + '"><i class="fas fa-external-link-alt"></i> ' + id + '</a>');

  html += '</div>'
  return html;

}

$('.kmg-dnb').click(function () {
  var e = $(this);
  if (e.data('loaded') !== true) {
    var id = e.data('id') + '';
    var ids = id.split(' ');
    var content = "";
    var deferreds = [];
    $.each(ids, function (index, pndid) {
      deferreds.push(
        $.get('https://ref.de.dariah.eu/pndsearch/json?id=' + pndid, function (res) {
          content += renderDNBData(res, pndid);
        })
      );
    });
    $.when.apply($, deferreds).then(function () {
      e.data('loaded', true);
      e.attr('data-content', content);

      var popover = e.data('bs.popover');
      popover.setContent();
      popover.$tip.addClass(popover.options.placement);
      var calculated_offset = popover.getCalculatedOffset(popover.options.placement, popover.getPosition(), popover.$tip[0].offsetWidth, popover.$tip[0].offsetHeight);
      popover.applyPlacement(calculated_offset, popover.options.placement);
    }).fail(function () {
      e.attr('data-content', 'Daten konnten nicht geladen werden');
      var popover = e.data('bs.popover');
      popover.setContent();
    });

  }

});

function renderDNBData(data, id) {
  var html = '<div class="gnd-table">'

  html += renderGNDDataRow('Name:', data.entry.preferredNameForThePerson);
  if (data.entry.si != null && Array.isArray(data.entry.si)) {
    var alternateNames = "";
    for (i = 1; i < data.entry.si.length; i++) {
      if (i == 4) {
        alternateNames += "[...]"
        break;
      }
      alternateNames += data.entry.si[i] + "<br/>";
    }
    html += renderGNDDataRow('Andere Namen:', alternateNames);
  }
  if (data.entry.variantNameForThePerson != null && Array.isArray(data.entry.variantNameForThePerson)) {
    var variants = "";
    for (i = 1; i < data.entry.variantNameForThePerson.length; i++) {
      if (i == 4) {
        variants += "[...]"
        break;
      }
      variants += data.entry.variantNameForThePerson[i] + "<br/>";
    }
    html += renderGNDDataRow('Varianten:', variants);
  }

  html += renderGNDDataRow('Weitere Angaben:', data.entry.biographicalInformation);
  html += renderGNDDataRow('Räumlicher Bezug:', data.entry.placeOfResidence);
  html += renderGNDDataRow('Geburtsdatum:', formatDate(data.entry.dateOfBirth));
  html += renderGNDDataRow('Geburtsort:', data.entry.placeOfBirth);
  html += renderGNDDataRow('Sterbedatum:', formatDate(data.entry.dateOfDeath));
  html += renderGNDDataRow('Sterbeort:', data.entry.placeOfDeath);
  html += renderGNDDataRow('DNB Eintrag:', '<a href="http://d-nb.info/' + id + '"><i class="fas fa-external-link-alt"></i> ' + id + '</a>');

  html += '</div>'
  return html;

}

function renderGNDDataRow(name, description) {
  var html = "";
  if (description != null) {
    html = '<div class="gnd-table-row">';
    html += '<div class="gnd-table-cell">';
    html += name;
    html += '</div>';
    html += '<div class="gnd-table-cell">';
    html += description;
    html += '</div>';
    html += '</div>';
  }
  return html;
}

function formatDate(date) {
  if (date == null)
    return null;
  /* before christ */
  if (date.startsWith('-'))
    return date;
  /* do not format year only */
  if (date.length == 4)
    return date;
  var fdate = "";
  var d = new Date(date);
  var day = d.getDate();
  if (day < 10)
    fdate = "0" + day + ".";
  else
    fdate = day + ".";
  var month = d.getMonth();
  month = month + 1;
  if (month < 10)
    fdate += "0" + month + ".";
  else
    fdate += month + ".";
  fdate += d.getFullYear();
  return fdate;
}

/* handle loading of lido information */
$('.kmg-art').click(function () {
  var e = $(this);
  if (e.data('loaded') !== true) {
    $.ajax({
      url: '/exist/apps/sade/textgrid/data/kmg-lido/' + e.data('id') + '.xml',
      dataType: 'html',
      success: function (data) {
        e.data('loaded', true);
        e.attr('data-content', data.replace(/\<(\?xml|(\!DOCTYPE[^\>\[]+(\[[^\]]+)?))+[^>]+\>/g, ''));
        var popover = e.data('bs.popover');
        popover.setContent();
        popover.$tip.addClass(popover.options.placement);
        var calculated_offset = popover.getCalculatedOffset(popover.options.placement, popover.getPosition(), popover.$tip[0].offsetWidth, popover.$tip[0].offsetHeight);
        popover.applyPlacement(calculated_offset, popover.options.placement);
      },
      error: function (jqXHR, textStatus, errorThrown) {
        e.attr('data-content', 'Daten konnten nicht geladen werden');
        var popover = e.data('bs.popover');
        popover.setContent();
      }
    });
  }
});


$('a[href="#goto"]').click(function(){
  var id = parseInt(this.id);
  var scrollto = $(".headlabel:eq('"+id+"')");
  $("html, body").animate({
      scrollTop: scrollto.offset().top - 115
  }, 750);
});

/* View in fullscreen */
var elem = document.documentElement;

function openFullscreen() {
if (elem.requestFullscreen) {
  elem.requestFullscreen();
} else if (elem.mozRequestFullScreen) {
  /* Firefox */
  elem.mozRequestFullScreen();
} else if (elem.webkitRequestFullscreen) {
  /* Chrome, Safari and Opera */
  elem.webkitRequestFullscreen();
} else if (elem.msRequestFullscreen) {
  /* IE/Edge */
  elem.msRequestFullscreen();
}
}

/* Close fullscreen */
function closeFullscreen() {
if (document.exitFullscreen) {
  document.exitFullscreen();
} else if (document.mozCancelFullScreen) {
  /* Firefox */
  document.mozCancelFullScreen();
} else if (document.webkitExitFullscreen) {
  /* Chrome, Safari and Opera */
  document.webkitExitFullscreen();
} else if (document.msExitFullscreen) {
  /* IE/Edge */
  document.msExitFullscreen();
}
}

function setGoldenHeight() {
  var marginNavbarTop = $("body header").css("height");
  if ($("body header").css("display") == "none") {
      marginNavbarTop = 0;
  }
  var marginEditionTopBar = $("#editionNavBar").css("height");
  var marginAdditional = 26;
  var margin = parseInt(marginNavbarTop)+parseInt(marginEditionTopBar)+parseInt(marginAdditional);
  var height = "calc(100vh - "+margin.toString()+"px)";
  $("#goldenlayout").css("height",height);
  myLayout.updateSize();
}

/* Fullscreen button: toggles top navbar and footer + container-fluid */
function triggerFullscreenButton() {
$('#icon-fullscreen svg').toggleClass("fa-expand");
$('#icon-fullscreen svg').toggleClass("fa-compress");

/* Hide/Show footer, top navbar and wrapper element */
var footer = document.getElementById("footer");
footer.style.display = footer.style.display === "none" ? "" : "none";
var navbar = document.getElementById("header");
navbar.style.display = navbar.style.display === "none" ? "" : "none";
}

function fullscreen() {
if ($('#icon-fullscreen svg').hasClass("fa-expand")) {
  openFullscreen();
} else {
  closeFullscreen();
}
if ($("#edition").length > 0) {
    setTimeout(function() {
          setGoldenHeight();
      }, 400);
}
}
if (document.addEventListener) {
document.addEventListener('webkitfullscreenchange', exitHandler, false);
document.addEventListener('mozfullscreenchange', exitHandler, false);
document.addEventListener('fullscreenchange', exitHandler, false);
document.addEventListener('MSFullscreenChange', exitHandler, false);
}

function exitHandler() {
if (document.webkitIsFullScreen === false) {
  triggerFullscreenButton();
} else if (document.mozFullScreen === false) {
  triggerFullscreenButton();
} else if (document.msFullscreenElement === false) {
  triggerFullscreenButton();
} else if (document.webkitIsFullScreen === true) {
  triggerFullscreenButton();
} else if (document.mozFullScreen === true) {
  triggerFullscreenButton();
} else if (document.msFullscreenElement === true) {
  triggerFullscreenButton();
}
}

function toggleLoader() {
var yourUl = document.getElementById("loader");
yourUl.style.display = yourUl.style.display === 'none' ? '' : 'none';
yourUl = document.getElementById("loaderWrapper");
yourUl.style.display = yourUl.style.display === 'none' ? '' : 'none';
}

/* Increase/Decrease Font-Size Buttons */
function getCurrentFontSize() {
  var currentFontSize;
if ($("#edition").length == 0) {
   currentFontSize = localStorage.getItem("fontSizeSurround");
}
else {
    currentFontSize = localStorage.getItem("fontSizeSynoptic");
}
return currentFontSize;
}
function setCurrentFontSize(input) {
if ($("#edition").length == 0) {
   localStorage.setItem("fontSizeSurround",input);
}
else {
    localStorage.setItem("fontSizeSynoptic",input);
}
}
function changeFont(change) {
var currentFontSize = getCurrentFontSize();
if ($("#edition").length == 0) {
   currentFontSize = localStorage.getItem("fontSizeSurround");
}
else {
    currentFontSize = localStorage.getItem("fontSizeSynoptic");
}
var newFontSize;
if (currentFontSize == null) {
  currentFontSize = "100"
}

newFontSize = (parseInt(currentFontSize) + change).toString();
setCurrentFontSize(newFontSize);
newFontSize = newFontSize + "%"
if ($("#text").length > 0) {
    $("#text").css("font-size", newFontSize);
}
}
function setFontSize() {
  try {
      var fontSize = getCurrentFontSize();
      fontSize = fontSize + "%"
      if ($("#text").length > 0) {
        $("#text").css("font-size", fontSize);
    }
  }
  catch (e) {console.log(e)}
}

function resetFont() {
  var defaultvalue;
  if ($("#edition").length > 0) {
      defaultvalue = 100;
  }
  else {
      defaultvalue = 100;
  }
  defaultvalue = defaultvalue + "%"
if ($("#text").length > 0) {
    $("#text").css("font-size", defaultvalue);
}
setCurrentFontSize(defaultvalue);
}

function increaseFont() {
changeFont(10);
}

function decreaseFont() {
changeFont(-10);
}
resetFont();

function loadBrowserHistory() {
updatePage();
}

$('header nav button.bars').on('click', function(event) {
event.preventDefault();
$('body').addClass('nav-overlay');
});

$('header nav button.cross').on('click', function(event) {
event.preventDefault();
$('body').removeClass('nav-overlay');
});

$('body #editionNavBar .bars').on('click', function(event) {
event.preventDefault();
$('body #editionNavBar .cross').removeClass('cross-invisible');
$('body #editionNavBar .bars').addClass('bars-invisible');
});

$('body #editionNavBar .cross').on('click', function(event) {
event.preventDefault();
$('body #editionNavBar .cross').addClass('cross-invisible');
$('body #editionNavBar .bars').removeClass('bars-invisible');
});

$('body #editionNavBar').ready(function () {
  $('[data-toggle="tooltip"]').tooltip();
});

$('.foldout .toggle').on('click', function(event) {
event.preventDefault();
let foldout = $(event.target).parents('.foldout');
foldout.toggleClass('show');
});

$('.search').on('click', 'a', function(event) {
event.preventDefault();
let input = $(event.target).parents('.search').find('input');
input.toggleClass('show');
input.focus();
});

$('.search').on('blur', 'input', function(event) {
let input = $(event.target);
if (input.val() === '') {
  input.removeClass('show');
}
});

/**
 * This function sets the content of a modal for editorial notes.
 * @param {id} id A HTML-id referring to the content to be included in the modal body.
 */
function setCorrModalContent(id) {
  var modalContent = document.getElementById(id).innerHTML;
  var modalBody = document.getElementById("corrModal").getElementsByClassName("modal-body")[0];
  modalBody.innerHTML = modalContent;
};

/**
 * This function sets the content of a modal for editorial notes.
 * @param {string} id A HTML-id referring to the content to be included in the modal body.
 */
function setNoteModalContent(id) {
  var modalContent = document.getElementById(id).innerHTML;
  var modalBody = document.getElementById("noteModal").getElementsByClassName("modal-body")[0];
  modalBody.innerHTML = modalContent;
};

/**
 * This function opens a modal displaying LIDO information on works of art.
 * @param {string} id A TG-URI referring to an object containing KMG LIDO data.
 */
function openartModal(id) {
    var modal = document.getElementById("artModal");
    var loading = modal.getElementsByClassName("loading")[0];
    var error = modal.getElementsByClassName("error")[0];
    var content = modal.getElementsByClassName("content")[0];
    loading.style.display = "block";
    content.style.display = "none";
    $(modal).modal("show");
    var xhttp = new XMLHttpRequest();
    xhttp.open("GET", "/exist/apps/sade/textgrid/data/kmg-lido/" + id + ".xml")
    xhttp.onreadystatechange = function() {
        if (this.readyState == 4 && this.status == 200) {
            modal.getElementByClassName("modal-body")[0].innerHtml = this.responseText;
        }
        else if (this.status !== 200) {
          loading.style.display = "none";
          error.style.display = "block";
      }
    }
    xhttp.send();
};

/**
 * This function opens a modal displaying information on periodicals.
 * @todo Write the function.
 */
function openbrandnameModal() {
  
};

/**
 * This function opens a modal displaying information on periodicals.
 * @todo Write the function.
 */
function openmusicModal() {
  
};

/**
 * This function opens a modal displaying information on organizations.
 * @param {string} A GND-id referring an organization. 
 */
function openorgModal(id) {
  var modal = document.getElementById("orgModal");
  var loading = modal.getElementsByClassName("loading")[0];
  var error = modal.getElementsByClassName("error")[0];
  var content = modal.getElementsByClassName("content")[0];
  loading.style.display = "block";
  content.style.display = "none";
  $(modal).modal("show");
  var xhttp = new XMLHttpRequest();
  xhttp.open("GET", "https://lobid.org/gnd/" + id + ".json", true);
  xhttp.onreadystatechange = function() {
      if (this.readyState == 4 && this.status == 200) {
          var object = JSON.parse(this.responseText);

          modal.getElementsByClassName("Modal__name")[0].innerHTML = object.preferredName;
          modal.getElementsByClassName("Modal__source")[0].firstElementChild.innerHTML = "http://d-nb.info/gnd/" + id;
          modal.getElementsByClassName("Modal__source")[0].setAttribute("href", "http://d-nb.info/gnd/" + id);
          loading.style.display = "none";
          content.style.display = "block";
      }
      else if (this.status !== 200) {
          loading.style.display = "none";
          error.style.display = "block";
      }
  }
  xhttp.send();
};

/**
 * This function opens a modal displaying information on periodicals.
 * @todo Write the function.
 */
function openperiodicalModal() {
  
};

/**
 * This function opens a modal displaying biographical data.
 * @param {string} id A GND-id referring a person.
 */
function openpersonModal(id) {
  var modal = document.getElementById("personModal");
  var loading = modal.getElementsByClassName("loading")[0];
  var error = modal.getElementsByClassName("error")[0];
  var content = modal.getElementsByClassName("content")[0];
  modal.getElementsByClassName("modal-content")[0].style = "";
  loading.style.display = "block";
  content.style.display = "none";
  $("#personModal").modal("show");
  var xhttp = new XMLHttpRequest();
  xhttp.open("GET", "https://ref.de.dariah.eu/pndsearch/json?id=" + id, true);
  xhttp.onreadystatechange = function() {
      if (this.readyState == 4 && this.status == 200) {
          var object = JSON.parse(this.responseText).entry;

          modal.getElementsByClassName("personModal__name")[0].innerHTML = object.preferredNameForThePerson;
          modal.getElementsByClassName("personModal__birth")[0].innerHTML = formatDate(object.dateOfBirth);
          modal.getElementsByClassName("personModal__death")[0].innerHTML = formatDate(object.dateOfDeath);
          modal.getElementsByClassName("personModal__further")[0].innerHTML = object.biographicalInformation;
          modal.getElementsByClassName("personModal__source")[0].firstElementChild.innerHTML = "http://d-nb.info/gnd/" + id;
          modal.getElementsByClassName("personModal__source")[0].setAttribute("href", "http://d-nb.info/gnd/" + id);
          loading.style.display = "none";
          content.style.display = "block";
      }
  }
  xhttp.send();
};

/**
 * This function opens a modal displaying bibliographical data.
 * @param {string} id An id of an object in the KMG Zotero-group.
 * @param {string} mode A string determining whether the modal displays editorial or authorial bibliographical data.
 */
function openrefModal(id, mode) {
  var modal = document.getElementById("refModal");
  var loading = modal.getElementsByClassName("loading")[0];
  var error = modal.getElementsByClassName("error")[0];
  var content = modal.getElementsByClassName("content")[0];
  loading.style.display = "block";
  content.style.display = "none";
  if (mode == "KMG") {
      modal.getElementsByClassName("modal-title-KMG")[0].style.display = "block";
      modal.getElementsByClassName("modal-title-base")[0].style.display = "none";
  }
  else {
      modal.getElementsByClassName("modal-title-KMG")[0].style.display = "none";
      modal.getElementsByClassName("modal-title-base")[0].style.display = "block";
  };
  $(modal).modal("show");
  var xhttp = new XMLHttpRequest();
  xhttp.open("GET", "https://staging.mollenhauer-edition.de/exist/restxq/zotero/items/" + id, true);
  xhttp.onreadystatechange = function() {
      if (this.readyState == 4 && this.status == 200) {
          var object = JSON.parse(this.responseText)[id];

          modal.getElementsByClassName("content")[0].innerHTML = object.bib;
          loading.style.display = "none";
          content.style.display = "block";
      }
      else if (this.status !== 200) {
          loading.style.display = "none";
          error.style.display = "block";
      }
  }
  xhttp.send();
};


/**
 * This function allows for the movement of modals by dragging the header.
 * @param {object} elmnt An element with the class "modal-content".
 */
const dragElement = function dragElement(elmnt) {
  var pos1 = 0, pos2 = 0, pos3 = 0, pos4 = 0;
  if (elmnt.getElementsByClassName("modal-header")[0]) {
    elmnt.getElementsByClassName("modal-header")[0].addEventListener("mousedown", dragMouseDown);
  } else {
    elmnt.addEventListener("mousedown", dragMouseDown);
  }

  function dragMouseDown(e) {
    e = e || window.event;
    e.preventDefault();
    pos3 = e.clientX;
    pos4 = e.clientY;
    document.addEventListener("mouseup", closeDragElement);
    document.addEventListener("mousemove", elementDrag);
  }

  function elementDrag(e) {
    e = e || window.event;
    e.preventDefault();
    pos1 = pos3 - e.clientX;
    pos2 = pos4 - e.clientY;
    pos3 = e.clientX;
    pos4 = e.clientY;
    elmnt.style.top = (elmnt.offsetTop - pos2) + "px";
    elmnt.style.left = (elmnt.offsetLeft - pos1) + "px";
  }

  function closeDragElement() {
    document.removeEventListener("mouseup", closeDragElement);
    document.removeEventListener("mousemove", elementDrag);
  }
}

var modals = document.getElementsByClassName("modal");
for (modal of modals) {
    dragElement(modal.getElementsByClassName("modal-content")[0]);
};